function [predL, predLL, AUC, FPR, TPR]=evaluateParcellation(z,Atrain,Atest,eta0)
% Usage:
%   [predL predLL AUC FPR TPR]=evaluateParcellation(z,Atrain,Atest,eta0)
%
% Input:
% z         I x 1 assignment vector providing the parcellation in terms of
%           node correspondence to parcels
% Atrain    I x I Training graph (binary and undirected - i.e. upper triangular)
% Atest     I x I Testting graph (binary and undirected - i.e. upper triangular)
% eta0      1 x 2 vector of prior link and non-link counts 
%           (used for calculating predL and predLL reliably, if not specified default eta0=[0.5 0.5] for predL and predLL and [1 1] for AUC)
% 
% Output:
% predL     predictive log expected likelihood of Atest when predicting using
%           Atrain
% predLL    predictive expected log-likelihood of Atest when predicting using
%           Atrain
% AUC       Area Under Curve when predicting Atest using Atrain
% FPR       False Positive Rate (used for AUC)
% TPR       True Positve Rate (used for AUC) 
%
% Written by Morten M?rup, 

if nargin<4
    eta0=[0.5 0.5];
end
% Make sure graph is upper triangular (undirected)
Atrain=triu(Atrain,1);
Atest=triu(Atest,1);
noc=max(z);
I=length(z);
Z=accumarray([z (1:I)'],ones(I,1),[noc I]);    

% Calculate sufficient statistics;
sumZ=sum(Z,2);
ZZT=sumZ*sumZ'-diag(sumZ);    

% Training data
ZAtrainZt=Z*Atrain*Z';
n_link_train=ZAtrainZt+ZAtrainZt';    
n_link_train=n_link_train-0.5*diag(diag(n_link_train));
n_link_wprior_train=n_link_train+eta0(1);
n_nonlink_train=ZZT-ZAtrainZt-ZAtrainZt';
n_nonlink_train=n_nonlink_train-0.5*diag(diag(n_nonlink_train));
n_nonlink_wprior_train=n_nonlink_train+eta0(2);

% Test data
ZAtestZt=Z*Atest*Z';    
n_link_test=ZAtestZt+ZAtestZt';    
n_link_test=n_link_test-0.5*diag(diag(n_link_test)); 
n_nonlink_test=ZZT-ZAtestZt-ZAtestZt';        
n_nonlink_test=n_nonlink_test-0.5*diag(diag(n_nonlink_test));                               

% Calculate predictive log of expected likelihood
logPilog=log(n_link_wprior_train)-log(n_link_wprior_train+n_nonlink_wprior_train);
log1mPilog=log(n_nonlink_wprior_train)-log(n_link_wprior_train+n_nonlink_wprior_train);    
predL=sum(sum(triu(n_link_test.*logPilog)))+sum(sum(triu(n_nonlink_test.*log1mPilog)));

% Calculate predictive expected log-likelihood
logPi=psi(n_link_wprior_train)-psi(n_link_wprior_train+n_nonlink_wprior_train);
log1mPi=psi(n_nonlink_wprior_train)-psi(n_link_wprior_train+n_nonlink_wprior_train);    
predLL=sum(sum(triu(n_link_test.*logPi)))+sum(sum(triu(n_nonlink_test.*log1mPi)));

if nargin<4 % Calculate AUC with uniform prior [1 1]
    eta=n_link_train./(n_link_train+n_nonlink_train);
else % Calculate AUC with prior
    eta=(n_link_wprior_train-1)./(n_link_wprior_train+n_nonlink_wprior_train-2);
end
idx=find(triu(ones(size(eta))));
[val,ind]=sort(eta(idx),'descend');
pos=n_link_test(idx);
neg=n_nonlink_test(idx);
val_old=-inf;
t=0;
positive=zeros(1,length(unique(val)));
negative=zeros(size(positive));        
for k=1:length(ind)
    if val_old~=val(k)
        t=t+1;
        val_old=val(k);
        positive(t)=0;
        negative(t)=0;
    end
    positive(t)=positive(t)+pos(ind(k));
    negative(t)=negative(t)+neg(ind(k));
end        
FPR=[0 cumsum(negative)/sum(negative)];
TPR=[0 cumsum(positive)/sum(positive)];
AUC = trapz(FPR,TPR);
    
